/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file corners.cpp
@ingroup ExamplePrograms
@brief Obtains <a href="http://en.wikipedia.org/wiki/Corner_detection#The_Harris_.26_Stephens_.2F_Plessey_corner_detection_algorithm">Harris corners</a> from the frames of a video input source.

@todo Detailed description should follow.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./corners --URL=<path or url to video>
\endcode

En mathematica:
H = {{h1, h2, h3}, {h4, h5, h6}, {h7, h8, h9}};
Cross[{p, q, 1}, H .{x, y, 1}]

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <math.h>

#include <QDebug>
#include <QVector>
#include <QThread>

#include <qvcore/qvapplication.h>
#include <qvcameras/qvmplayercamera.h>
#include <qvgui/qvgui.h>

#include <qvdta/qvpolyline.h>
#include <qvdta/qvdta.h>

#include <TooN/TooN.h>
#include <TooN/numerics.h>	// Basic TooN library
#include <TooN/numhelpers.h>	// If you want a few extras
#include <TooN/SVD.h>		// If you want singular value decompositions
#include <TooN/LU.h>		// If you want lu triangular decompositions
#include <TooN/SymEigen.h>	// If you want Eigen decomposition of symmetric matrices

#define PI	3.1415926535

Q_DECLARE_METATYPE(Matrix<>);

/// \cond

/////////////////////////////////////////////////////////////////////////////////////////////////////

/*#include <GL/glut.h>
#include <math.h>
float phi, theta;
//static float angle;
int keyboard;
int last_x, last_y;

double x, y, z;
GLfloat rot[16] = {1, 0, 0, 0,  0, 1, 0, 0,  0, 0, 1, 0,  0, 0, 0, 1};
GLfloat tra[16] = {1, 0, 0, -2,  0, 1, 0, 0,  0, 0, 1, -6,  0, 0, 0, 1};
GLfloat rot45[16] = {0.707, -0.707, 0, 0,  0.707, 0.707, 0, 0,  0, 0, 1, 0,  0, 0, 0, 1}; 

void init(void) {
	// white ambient light
	GLfloat lightPosition[] = {30.0f, 0.0f, 3.0f, 0.0f};
	GLfloat shininess[] = {50};
	GLfloat whiteLight[] = {1.0f, 1.0f, 1.0f, 1.0f};
	GLfloat ambient[] = {0.1f, 0.1f, 0.1f, 0.1f};
	GLfloat specular[] = {1.0f, 1.0f, 1.0f, 1.0f};
	//glLightModelfv(GL_FRONT, ambientLight);
	glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, shininess);
	//glLightModelfv(GL_FRONT, GL_SHININESS, specular);
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, whiteLight);
	glLightfv(GL_LIGHT0, GL_SPECULAR, whiteLight);
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_SMOOTH);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	//glEnable(GL_DEPTH_TEST);
}

// Drawn Ground

void DrawGround(void) {
	GLfloat fExtent = 20.0;
	GLfloat fStep = 1.0f;
	GLint y = -2.4f;// Ground level 
	GLint iLine;
	glBegin(GL_LINES);
		for(iLine= -fExtent; iLine <= fExtent; iLine += fStep) {
			// Draw z lines
			glVertex3f(iLine, y, fExtent);
			glVertex3f(iLine, y, -fExtent);
			// Draw x lines
			glVertex3f(fExtent, y, iLine);
			glVertex3f(-fExtent, y, iLine);
		}
	glEnd();
}
			
void display(void) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glColor3f(1.0, 1.0, 1.0);
	
	glPushMatrix();
	//gluLookAt(0, 0, 5, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
	glTranslatef(0, 0, -5);
	glMultTransposeMatrixf(rot45);
	glutSolidCube(1);
	//glMatrixMode(GL_MODELVIEW);
	
	glPopMatrix();
	

	//glTranslatef(0.0, 0.0, -5.0);
	
	//glRectf(-25.0, -25.0, 25.0, 25.0);
	glLoadIdentity();
	gluLookAt(0, 0, 8, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
	glRotatef(-phi, 0, 0, 1);
	glRotatef(theta, 0, 1, 0);
	//glTranslatef(0, -5.0f, 0);		
	

	
	DrawGround();
	glutSwapBuffers();
}

void reshape(int w, int h) {
	GLfloat fAspect;
    	// Prevent a divide by zero, when window is too short
    	if(h == 0)
        	h = 1;
    	glViewport(0, 0, w, h);
    	fAspect = (GLfloat)w / (GLfloat)h;
    	glMatrixMode(GL_PROJECTION);
	// Reset the coordinate system
    	glLoadIdentity();
    	// Set the clipping volume
    	gluPerspective(60.0f, fAspect, 1.0f, 50.0f);
    	glMatrixMode(GL_MODELVIEW);
    	glLoadIdentity();
}

void spin(void) {
	//angle += 90;
	//if (angle > 360) angle = angle -360;
	glutPostRedisplay();
}

void mouse(int x, int y) {
	// Capture the keyboard button
	keyboard = glutGetModifiers();
	// If both a left mouse botton and ALT key are pressed then do
	if  (keyboard == GLUT_ACTIVE_CTRL) {
			theta += (x - last_x)*0.25f; 
			phi += (y - last_y)*0.25f;
			if (phi < -20.0f) phi = -20.0f;
			else if (phi > 80.0f) phi = 80;
			last_x = x;
			last_y = y;
	}
}

void keyb(unsigned char key, int x, int y) {
	switch(key) {
		case 'd' :
			theta += (x - last_x)*0.0025f; 
			glutPostRedisplay();
			break;
		case 'D' :
			phi += (y - last_y)*0.0025f;
			if (phi < -20.0f) phi = -20.0f;
			else if (phi > 80.0f) phi = 80;
			last_x = x;
			last_y = y;
			glutPostRedisplay();
			break;
		case 'y' :
			//year= (year +5) % 360;
			glutPostRedisplay();
			break;
		case 'Y' :
			//year= (year -5) % 360;
			glutPostRedisplay();
			break;
	}
}	

class OpenGLThread: public QThread
	{
	public:
		OpenGLThread(int argc, char **argv, QObject * parent = 0):QThread(parent), argc(argc), argv(argv)	{ }

		void run()
			{
			glutInit(&argc, argv);
			glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
			glutInitWindowSize(800, 600);
			//glutInitWindowPosition(100,100);
			// Create the window
			glutCreateWindow(argv[0]);
			// Initialize rendering
			init();
			// Whenever GLUT determines that the contents of the window needs to be redisplayed,
			// the callback function registered by glutDisplayFunc() is executed
			glutDisplayFunc(display);
			// Determines what action should be taken when the windows is resized
			glutReshapeFunc(reshape);
			// Determines the action when the mouse is moved or a botton is pressed
			glutKeyboardFunc(keyb);
			glutMainLoop();
			}
	private:
		int argc;
		char **argv;
	};*/

/////////////////////////////////////////////////////////////////////////////////////////////////////

void GetHotPoints(const QVImage<sFloat> cornerResponseImage, QList<QPoint> &hotPoints, uInt maxWindow)
	{
	const uInt rows = cornerResponseImage.getRows(), cols = cornerResponseImage.getCols();

	QVImage<uChar> binaryCornerImage(cols, rows);
	FilterLocalMax(cornerResponseImage, binaryCornerImage, maxWindow, maxWindow);

	QVIMAGE_INIT_READ(uChar,binaryCornerImage);
	for (uInt row = 0; row < binaryCornerImage.getRows(); row++)
		for (uInt col = 0; col < binaryCornerImage.getCols(); col++)
			if (QVIMAGE_PIXEL(binaryCornerImage, col, row,0))
				hotPoints.append(QPoint(col, row));
	}

void GetMaximalPoints(const QVImage<sFloat> cornerResponseImage, QList<QPoint> &hotPoints, QList<QPoint> &maximalPoints, uInt maxPoints)
	{
	while( hotPoints.size() > 0 && maximalPoints.size() < maxPoints )
		{
		uInt maxIndex = 0;
		for (int n=0; n < hotPoints.size(); n++)
			if ( cornerResponseImage(hotPoints.at(n)) >  cornerResponseImage(hotPoints.at(maxIndex)) )
				maxIndex = n;

		maximalPoints.append(hotPoints.at(maxIndex));
		hotPoints.removeAt(maxIndex);
		}
	}

uInt	getClosestPointIndex(const QPoint point, const QList<QPoint> &pointList)
	{
	uInt index = 0;
	for (uInt n = 1; n < pointList.size(); n++)
		if ((point - pointList.at(n)).manhattanLength() < (point - pointList.at(index)).manhattanLength())
			index = n;

	return index;
	}

QPoint	getMeanPoint(const QList<QPoint> &pointList)
	{
	QPoint center(0,0);

	for (uInt n = 0; n < pointList.size(); n++)
		{
		center.rx() += pointList.at(n).x();
		center.ry() += pointList.at(n).y();
		}

	center.rx() = center.rx() / pointList.size();
	center.ry() = center.ry() / pointList.size();

	return center;
	}

double	angle(const QPoint &p)
	{
	double x = p.x(), y = p.y();
	if (x>0)
		if (y>=0)
			return atan(y/x);
		else
			return atan(y/x) + 2*PI;
	else if (x == 0)
		if (y>0)
			return PI/2;
		else
			return 3*PI/2;
	else // x < 0
		return atan(y/x)+PI;
	}

double	clockWiseAngle(const QPoint &p1, const QPoint &p2)
	{
	double clockAngle = angle(p2) - angle(p1);
	return (clockAngle < 0)? clockAngle + 2*PI:clockAngle;
	}

// This is suposed to sort the points in a list, by... who knows?
// Anyway, it doesn't works
bool SortTemplatePoints(QList<QPoint> &points)
	{
	if (points.size() != 5)
		return false;

	QList<QPoint> result;

	// This array will keep the index of the points.
	uInt index[5];

	// Calculate the index of the central point
	uInt indexp = getClosestPointIndex(getMeanPoint(points), points);

	result.append(points.at(indexp));
	points.removeAt(indexp);

	// Calculate the index of the top left point
	double minDistance = 1000000;
	for (uInt n = 0; n < points.size(); n++)
		if ( points.at(n).manhattanLength() < minDistance )
			{
			minDistance = points.at(n).manhattanLength();
			indexp = n;
			}

	result.append(points.at(indexp));
	points.removeAt(indexp);

	// Calculate the other points
	while(points.size() > 0)
		{
		indexp = 0;
		double minAngle = clockWiseAngle( result.back() - result.front(), points.at(indexp) - result.front());
	
		for (uInt n = 1; n < points.size(); n++)
			{
			double actualAngle = clockWiseAngle( result.back() - result.front(), points.at(n) - result.front());
			if ( actualAngle < minAngle )
				{
				minAngle = actualAngle;
				indexp = n;
				}
			}

		result.append(points.at(indexp));
		points.removeAt(indexp);
		}

	points = result;

	return true;
	}

// This function returns the homography that maps the points from a source position to a
// destination position
Matrix<> CalibrateHomography(const Matrix<> &sourcePointsMatrix, const Matrix<> &destinationPoints)
	{
	Q_ASSERT(sourcePointsMatrix.num_cols() == 2);
	Q_ASSERT(sourcePointsMatrix.num_cols() == destinationPoints.num_cols());
	Q_ASSERT(sourcePointsMatrix.num_rows() == destinationPoints.num_rows());

	const uInt rows = sourcePointsMatrix.num_rows();

	// First, state the coefs matrix for the homogeneous linear system of equations, of the form:
	//	Ax = 0
	Matrix<> coefsMatrix(3*rows,9);

	for (uInt n = 0; n < rows; n++)
		{
		double	x = sourcePointsMatrix[n][0], y = sourcePointsMatrix[n][1],
			p = destinationPoints[n][0], q = destinationPoints[n][1];

		double	equation1[9] = {	0,	0,	0,	-x,	-y,	-1,	q*x,	q*y,	q},
			equation2[9] = {	x,	y,	1,	0,	0,	0,	-p*x,	-p*y,	-p},
			equation3[9] = {	-q*x,	-q*y,	-q,	p*x,	p*y,	p,	0,	0,	0};

		coefsMatrix[3*n] = Vector<9>(equation1);
		coefsMatrix[3*n+1] = Vector<9>(equation2);
		coefsMatrix[3*n+2] = Vector<9>(equation3);
		}

	// create the SVD of M
	SVD<> svdCoefsMatrix(coefsMatrix);

	Vector<9> x =  svdCoefsMatrix.get_VT()[8];
	Matrix<> homography(3,3);

	homography[0][0] = x[0];	homography[0][1] = x[1];	homography[0][2] = x[2];
	homography[1][0] = x[3];	homography[1][1] = x[4];	homography[1][2] = x[5];
	homography[2][0] = x[6];	homography[2][1] = x[7];	homography[2][2] = x[8];

	return homography;
	}

void normalizeHomogeneousCoordinates(Matrix<> &points)
	{
	const uInt cols = points.num_cols(), rows = points.num_rows();

	for (uInt i = 0; i < rows; i++)
		for (uInt j = 0; j < cols; j++)
			points[i][j] /= points[i][cols-1];
	}

void normalizeHomogeneousCoordinates(Matrix<1,3> &points)
	{
	const uInt cols = points.num_cols(), rows = points.num_rows();

	for (uInt i = 0; i < rows; i++)
		for (uInt j = 0; j < cols; j++)
			points[i][j] /= points[i][cols-1];
	}

// This function retunrs an integer value that indicates how well the homography maps the source points
// in a list, to the points in the destination list.
double	testErrorHomography(const Matrix<> &sourcePointsMatrix, const Matrix<> &destinationPoints, const Matrix<> homography)
	{
	const uInt cols = sourcePointsMatrix.num_cols(), rows = sourcePointsMatrix.num_rows();

	Matrix <> projectedPoints(sourcePointsMatrix.num_rows(),3);
	Matrix <> residuals (sourcePointsMatrix.num_rows(),3);

	projectedPoints = sourcePointsMatrix * homography.T();
	normalizeHomogeneousCoordinates(projectedPoints);
	residuals = projectedPoints - destinationPoints;

	double accum = 0;
	for (uInt i = 0; i < rows; i++)
		{
		double square = 0;
		for (uInt j = 0; j < cols; j++)
			square += residuals[i][j]*residuals[i][j];
		accum += sqrt(square);
		}

	return accum;
	}

// This function creates a list of the template points, first with the center point,
// then the rest of the points following a clockwise order around the center point.
Matrix<> GetTemplateMatrixPoints()
	{
	//					Center		Norwest		Noreast		Southeast	Southwest
	return Matrix<5,3> ((double[5][3]){	0,0,1,		-1,+1,+1,	+1,+1,+1,	+1,-1,+1,	-1,-1,+1});
	}

// This function sustitudes warpPerspective from the qvipp, and applies the planar transformation H
// to a source image, storing the transformed image in the destination image
void	myWarpPerspective(const QVImage<uChar> &src, QVImage<uChar> &dest, const Matrix <> H, const double zoom)
	{
	const double cols = src.getCols(), rows = src.getRows();
	const Matrix<> Hinv = SVD<>(H).get_pinv();

	for (double col = 0; col < cols; col++)
		for (double row = 0; row < rows; row++)
			{
			const double x = col, y = row;

			double	v[3] = { 2*(x - cols/2)/cols, -2*(y - rows/2)/cols, 1 };

			Vector <3> vP;
			vP = H * Vector<3>(v);
			const double x0 = vP[0]/vP[2], y0 = vP[1]/vP[2];
			const QPoint p2 =  QPoint( (uInt) x, (uInt) y ), p1 = QPoint( zoom*x0 + cols/2, -zoom*y0 + rows/2 );

			if (dest.getROI().contains(p2) && src.getROI().contains(p1))
				dest(p1) = src(p2);
			}
	}

// This function transforms a list of n points, to a nx3 matrix, containing the rows the
// homogeneous coordinates for each point in the list.
void pointListToMatrix(const QVImage<uChar> &image, const QList<QPoint> &points, Matrix<> &matrix)
	{
	Q_ASSERT(points.size() == matrix.get_rows());
	Q_ASSERT(3 == matrix.get_cols());
	const double	rows = image.getRows(), cols = image.getCols();

	for (uInt n = 0; n < points.size(); n++)
		{
		const double p = points.at(n).x(), q = points.at(n).y();
		double	v[3] = { 2*(p - cols/2)/cols, -2*(q - rows/2)/cols, 1 };
		matrix[n] = Vector<3>(v);
		}
	}

// Gets the minimum point of the parabol that contains the three points (x1,y1), (x2,y2), (x3,y3). In Mathematica:
// Solve[(2a x + b /. Solve[{	a x1^2 + b  x1 + c == y1,
//				a x2^2 + b  x2 + c == y2,
//				a x3^2 + b  x3 + c == y3	},
//	{a, b, c}]) == 0, x]
double minimumAtCuadraticInterpolation(const double x1, const double y1, const double x2, const double y2, const double x3, const double y3)
	{
	const double	numerator = x2*x2*y1 - x3*x3*y1 - x1*x1*y2 + x3*x3*y2 + x1*x1*y3 - x2*x2*y3,
			denominator =  x2*y1 - x3*y1 - x1*y2 + x3*y2 + x1*y3 - x2*y3;

	return numerator/(2*denominator);
	}

#define	POW2(X)	((X)*(X))


// Error function, given an 'f', and a 'H' matrix. In Mathematica:
//	K = {{f, 0, 0}, {0, f, 0}, {0, 0, 1}};
//	Hvar = {{h1, h2, h3}, {h4, h5, h6}, {h7, h8, h9}};
//	ErrorMat[H_, K_] := Transpose[H].Inverse[K].Inverse[K].H;
//	Error [ErrorM_] := ((ErrorM[[1, 1]] - ErrorM[[2, 2]])^2 + ErrorM[[1, 2]]^ 2) / ErrorM[[1, 1]]^ 2;
//	ErrorFunction[H_, K_] := Error[ErrorMat[H, K]];
//	ErrorFunction[Hvar, K] // Simplify
#define	ERROR(f) 								\
	(	(								\
		POW2( h1*h2 + h4*h5 + (f)*(f)*h7*h8 ) +				\
		POW2( h1*h1 - h2*h2 + h4*h4 - h5*h5 + (f)*(f)*(h7*h7-h8*h8) )	\
		) / POW2(h1*h1 + h4*h4 + (f)*(f)*h7*h7)				\
	)
double errorHomographyFocalDistance(const Matrix <3,3> &H, const double f)
	{
	const double	h1 = H[0][0],	h2 = H[0][1],	h3 = H[0][2],
			h4 = H[1][0],	h5 = H[1][1],	h6 = H[1][2],
			h7 = H[2][0],	h8 = H[2][1],	h9 = H[2][2];
	return ERROR(f);
	}

double errorHomographiesFocalDistance(const QVector< Matrix <3,3> > &Hlist, const double f)
	{
	double error = 0;
	for (uInt n = 0; n< Hlist.size(); n++)
		{
		Matrix<3,3> H = Hlist[n];
		const double	h1 = H[0][0],	h2 = H[0][1],	h3 = H[0][2],
				h4 = H[1][0],	h5 = H[1][1],	h6 = H[1][2],
				h7 = H[2][0],	h8 = H[2][1],	h9 = H[2][2];
		error +=	( ( POW2( h1*h2 + h4*h5 + (f)*(f)*h7*h8 ) + POW2( h1*h1 - h2*h2 + h4*h4 - h5*h5 + (f)*(f)*(h7*h7-h8*h8) ) ) / POW2(h1*h1 + h4*h4 + (f)*(f)*h7*h7) );
		}

	return error;
	}

// This function finds the fov that minimices errorHomographyFocalDistance for a given homography H,
// in a neighbourhood of a fov value passed in the variable 'startingFov'.
double calibrateFovDistance(const Matrix<> &Horig, const double startingFov = 3)
	{
	Matrix <3,3> H;

	// We will work with the inverse of Horig
	H = SVD<>(Horig).get_pinv();
	double f = startingFov;
	
	for (uInt i = 0; i < 10; i++)
		{
		f = minimumAtCuadraticInterpolation(
					f, errorHomographyFocalDistance(H,f),
					f - 0.1, errorHomographyFocalDistance(H,f - 0.1),
					f + 0.1, errorHomographyFocalDistance(H,f + 0.1)
					);
		}
	//std::cout << i << " f = " << f << ", ERROR = " << errorHomographyFocalDistance(H,f) << std::endl;
	return f;
	}

bool minimizeFovErrorFunction(const QVector< Matrix<3,3> > &H, double &f, const uInt maxSteps = 10)
	{
	double startError = errorHomographiesFocalDistance(H,f);
	double lastF = f, lastError = startError;

	for (uInt i = 0; i < maxSteps; i++)
		{
		f = minimumAtCuadraticInterpolation(
					f, errorHomographiesFocalDistance(H,f),
					f - 0.1, errorHomographiesFocalDistance(H,f - 0.1),
					f + 0.1, errorHomographiesFocalDistance(H,f + 0.1)
					);

		const double error = errorHomographiesFocalDistance(H,f);

		//std::cout << i << " f = " << f << ", ERROR = " << error << std::endl;

		if ( error < 0 || error > startError)
			return false;
		if (error < 0)
			return false;

		if (error > lastError)
			{
			f = lastF;
			return true;
			}

		lastF = f;
		lastError = error;
		}

	return false;
	}

double calibrateFovDistance(const QVector< Matrix<3,3> > &Horig, const uInt maxSteps = 10)
	{
	QVector< Matrix<3,3> > Hlist;
	Hlist.resize(Horig.size());

	
	// We will work with the inverse of Horig's
	for(uInt n=0; n< Horig.size(); n++)
		Hlist[n] = SVD<>(Horig[n]).get_pinv();

	for (double f0 = 0.5; f0 < 10; f0 += 0.5)
		{
		double f = f0;
		if (minimizeFovErrorFunction(Hlist, f, maxSteps))
			return f;
		}

	return 0.0;
	}

void crossProduct(const double x1, const double y1, const double z1, const double x2, const double y2, const double z2, double &x, double &y, double &z)
	{
	x = -y2*z1 + y1*z2;
	y = x2*z1 - x1*z2;
	z = -x2*y1 + x1*y2;
	};

void crossProduct(const Vector<3> v1, const Vector<3> v2, Vector<3> &v)
	{
	const double	x1 = v1[0], y1 = v1[1], z1 = v1[2],
			x2 = v2[0], y2 = v2[1], z2 = v2[2];

	v[0] = -y2*z1 + y1*z2;
	v[1] = x2*z1 - x1*z2;
	v[2] = -x2*y1 + x1*y2;
	};


/*void decomposeHomography(const double &f, const Matrix<3,3> &Horig)
	{
	const double m[9] = {1/f,0,0,0,1/f,0,0,0,1};
	Matrix<3,3> K(m);
	Matrix<3,3> HE,H;

	H = SVD<>(Horig).get_pinv();
	HE = K*H;

	Vector<3> R1, R2, R3, T;

	R1 = HE.T()[0];
	R2 = HE.T()[1];
	T = HE.T()[2];

	crossProduct(R1, R2, R3);
	//std::cout << "R1 = " << R1 << std::endl;
	//std::cout << "R2 = " << R2 << std::endl;
	//std::cout << "R3 = " << R3 << std::endl;

	Matrix<3,3> R, Rinv;

	R.T()[0] = R1;
	R.T()[1] = R2;
	R.T()[2] = R3;

	Rinv = SVD<>(R).get_pinv();

	const double	normR1 = sqrt(R1[0]*R1[0] + R1[1]*R1[1] + R1[2]*R1[2]),
			normR2 = sqrt(R2[0]*R2[0] + R2[1]*R2[1] + R2[2]*R2[2]),
			normR3 = sqrt(R3[0]*R3[0] + R3[1]*R3[1] + R3[2]*R3[2]);
			
	R3[0] = sqrt((normR1+normR2)/2)* R3[0] /normR3;
	R3[1] = sqrt((normR1+normR2)/2)* R3[1] /normR3;
	R3[2] = sqrt((normR1+normR2)/2)* R3[2] /normR3;

	Matrix<3,4> F;
	F.T()[0] = R1;
	F.T()[1] = R2;
	F.T()[2] = R3;
	F.T()[3] = T;

	Vector<3> C;

	C = (-1) * Rinv.T() * T;

	std::cout << "C = " << std::endl << C << std::endl;
	std::cout << " R1xR1 = " << (R1[0]*R1[0] + R1[1]*R1[1] + R1[2]*R1[2]) << std::endl;
	std::cout << " R2xR2 = " << (R2[0]*R2[0] + R2[1]*R2[1] + R2[2]*R2[2]) << std::endl;
	std::cout << " R3xR3 = " << (R3[0]*R3[0] + R3[1]*R3[1] + R3[2]*R3[2]) << std::endl;

	std::cout << " R1xR2 = " << (R1[0]*R2[0] + R1[1]*R2[1] + R1[2]*R2[2]) << std::endl;
	std::cout << " R1xR3 = " << (R1[0]*R3[0] + R1[1]*R3[1] + R1[2]*R3[2]) << std::endl;
	std::cout << " R2xR3 = " << (R2[0]*R3[0] + R2[1]*R3[1] + R2[2]*R3[2]) << std::endl;
	std::cout << " CxC = " << (C[0]*C[0] + C[1]*C[1] + C[2]*C[2]) << std::endl;

	std::cout << "Coors = (" << C[0]/C[2] << ", " << C[1]/C[2] << ")" << std::endl;

	//std::cout << " distance = " << (t[0]*t[0] + t[1]*t[1] + t[2]*t[2]) << std::endl;
	
	}*/

#define	SCALAR_PRODUCT(V1, V2)	sqrt((V1)[0]*(V2)[0] + (V1)[1]*(V2)[1] + (V1)[2]*(V2)[2])
#define	ABS(X)	((X>0)?(X):(-X))
void decomposeHomography(const double &f, const Matrix<3,3> &Horig, double &x, double &y, double &alpha, double &beta, double &gamma)
	{
	// O se coge H -> filas, o se coge Hinversa -> columnas
	// Valor positivo de f, valor negativo de f
	const double m[9] = {1/ABS(f),0,0,0,1/ABS(f),0,0,0,1};
	Matrix<3,3> Kinv(m);
	Matrix<3,3> R_Rt,H;

	//H = Horig;
	H = SVD<>(Horig).get_pinv();

	// Esta matriz es la que corresponde al segundo multiplicando del lado derecho de la expresión:
	//
	//	H = K[R|-Rt]
	//
	R_Rt = Kinv*H;

	Vector<3> v1 = R_Rt.T()[0], v2 = R_Rt.T()[1];

	// Homogenize R_Rt such as the product of the matrix M by its traspose has identity in
	// the 3x3 top left matrix. DUDA: esto se debería hacer antes, con la matrix H, o
	// ahora con la matriz R_Rt
	R_Rt /= (SCALAR_PRODUCT(v1, v1) + SCALAR_PRODUCT(v2, v2)) / 2;

	Vector<3> R1, R2, R3, Rt;

	R1 = R_Rt.T()[0];
	R2 = R_Rt.T()[1];
	Rt = R_Rt.T()[2];

	crossProduct(R1, R2, R3);

	Matrix<3,4> M;

	M.T()[0] = R1;
	M.T()[1] = R2;
	M.T()[2] = R3;
	M.T()[3] = Rt;

	std::cout << "M = " << std::endl << M << std::endl;

	Matrix<4,4> Mult;

	Mult = M.T() * M;

	// Position is at the first tree elements of last col of the following matrix
	std::cout << "M^T * M = " << std::endl << Mult << std::endl;
	}

#define	MAX_BEST_MATRICES	12
class MyWorker: public QVWorker
	{
	double	matrixError[MAX_BEST_MATRICES];
	QVector< Matrix<3,3> > matrixList;

	public:
		MyWorker(QString name): QVWorker(name)
			{
			matrixList.resize(MAX_BEST_MATRICES);
			for(uInt n=0; n<MAX_BEST_MATRICES; n++)
				matrixError[n] = 100;

			addProperty<double>("Max error", inputFlag, 0.02, "for an homography to be considered good", 0, 0.1);
			addProperty<double>("Zoom", inputFlag, 50, "Size of the rectified template", 1, 100);

			addProperty<double>("Focal", outputFlag, 0, "Focal distance");

			addProperty<int>("Window size", inputFlag, 10, "Corner response window search", 1, 100);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,3> >("Corners", outputFlag);
			addProperty< QVImage<uChar,1> >("Wrapped", outputFlag);

			addTrigger("Calibrate");
			addTrigger("Locate");
			addProperty< Matrix<> >("Actual homography", outputFlag);
			}

		void processTrigger(QString triggerName)
			{
			const Matrix<3,3> H = getPropertyValue< Matrix<> > ("Actual homography");

			if (triggerName == "Calibrate")
				{
				/*std::cout <<	"H = {"
						<< "{" << H[0][0] << ", " << H[0][1] << ", " << H[0][2] << "},"
						<< "{" << H[1][0] << ", " << H[1][1] << ", " << H[1][2] << "},"
						<< "{" << H[2][0] << ", " << H[2][1] << ", " << H[2][2] << "}};" << std::endl;
	
				std::cout << "--------- Best homographies ----------" << std::endl;
				for (uInt n=0; n<MAX_BEST_MATRICES; n++)
					{
					Matrix<3,3> H;
					H = matrixList[n];
					std::cout <<	"H" << n <<" = {"
						<< "{" << H[0][0] << ", " << H[0][1] << ", " << H[0][2] << "},"
						<< "{" << H[1][0] << ", " << H[1][1] << ", " << H[1][2] << "},"
						<< "{" << H[2][0] << ", " << H[2][1] << ", " << H[2][2] << "}}; (* " << matrixError[n] << "*)" << std::endl;
					}*/
	
				double fov = calibrateFovDistance(matrixList);
				std::cout << "FOV[" << MAX_BEST_MATRICES << "] (more stable) = " << fov << std::endl;
				std::cout << "FOV = " << calibrateFovDistance(getPropertyValue< Matrix<> >("Actual homography"),fov+1) << std::endl;
				setPropertyValue<double>("Focal",fov);
				}
			else if (triggerName == "Locate")
				{
				const double fov = getPropertyValue<double>("Focal");
				std::cout << "----------------------------" << std::endl;
				double x, y, alpha, beta, gamma;
				if (fov != 0)
					decomposeHomography(fov, H, x, y, alpha, beta, gamma);
				}
			};

		void iterate()
			{
			const QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			const uInt	rows = image.getRows(), cols = image.getCols(),
					sizeMax = getPropertyValue<int>("Window size");

			const double	maxError = getPropertyValue<double>("Max error"),
					zoom = getPropertyValue<double>("Zoom"),
					focal = getPropertyValue<double>("Focal");

			QVImage<uChar,3> destino = image;

			timeFlag("grab Frame");

			////////////////////////////////////
			// Harris corner response image
			QVImage<sFloat>	temp(cols, rows), cornerResponseImage(cols, rows);
			//HarrisCornerResponseImage(image, cornerResponseImage);
			SobelCornerResponseImage(image, cornerResponseImage);

			timeFlag("Corner response image");

			////////////////////////////////////
			// Hot points
			QList<QPoint> hotPoints;
			GetHotPoints(cornerResponseImage, hotPoints, sizeMax);

			timeFlag("Get hotpoints");

			///////////////////////////////////////////
			// Calibración
			QList<QPoint> maximalPoints;
			GetMaximalPoints(cornerResponseImage, hotPoints, maximalPoints, 5);
			SortTemplatePoints(maximalPoints);

			drawPoints(maximalPoints, destino);

			//std::cout << "----------------------------" << std::endl;

			//for (uInt i=0; i< maximalPoints.size(); i++)
			//	std::cout << "MaximalPoint (" << maximalPoints.at(i).x() << ", " << maximalPoints.at(i).y() << "), value = " << (double)cornerResponseImage(maximalPoints.at(i)) << std::endl;

			timeFlag("Get max hotpoints");

			if (maximalPoints.size() == 5)
				{
				Matrix <> sourcePointsMatrix(5,3), destinationPointsMatrix(5,3);
				destinationPointsMatrix =  GetTemplateMatrixPoints();
				pointListToMatrix(image, maximalPoints, sourcePointsMatrix);

				Matrix<> H = CalibrateHomography(sourcePointsMatrix, destinationPointsMatrix);

				const double actualError = testErrorHomography(sourcePointsMatrix, destinationPointsMatrix, H);
				if (actualError < maxError)
					{
					setPropertyValue< Matrix<> >("Actual homography", H);

					uInt index = 0;
					for (uInt i=1; i<MAX_BEST_MATRICES; i++)
						if (matrixError[i] > matrixError[index])
							index = i;

					if (matrixError[index] > actualError)
						{
						matrixList[index] = H;
						matrixError[index] = actualError;
						}

					QVImage<uChar> wrapped(cols, rows);
					Set(wrapped,0);
	
					myWarpPerspective(image, wrapped, H, zoom);

					setPropertyValue< QVImage<uChar,1> >("Wrapped", wrapped);

					}
				//std::cout << "----------------------------" << std::endl;
				//std::cout << "H = " << std::endl << H << std::endl;
				//std::cout << "Hinv = " << std::endl << SVD<>(H).get_pinv() << std::endl;

				//setPropertyValue<double>("Focal",0);
				}

			timeFlag("Calibrate");

			// Fin rectificación ////////////////////////

			setPropertyValue< QVImage<uChar,3> >("Corners", destino);
			timeFlag("Draw corners");
		
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		
		"Example program for QVision library. Applies corner detection over an input video."

		);

	/////////////////////////////////////
	OpenGLThread openGLThread(argc, argv);
	openGLThread.start();
	
	/////////////////////////////////////

	QVMPlayerCamera camera("Video");
	MyWorker worker("Corners Worker");
	camera.link(&worker,"Input image");

	QVGUI interface;

	QVImageCanvas imageCanvas("Corners");
	imageCanvas.linkProperty(worker, "Corners");

	QVImageCanvas imageCanvas2("Wrapped");
	imageCanvas2.linkProperty(worker, "Wrapped");

	return app.exec();
	}

/// \endcond

